@@ -1,5 +1,6 @@
 from instrumentDialog import Ui_instrumentDialog
 from mingus.containers.Instrument import MidiInstrument
+from mingus.core.notes import int_to_note
 from PyQt4 import QtCore, QtGui
 import Options
 import Musicians
@@ -31,6 +32,11 @@ class InstrumentDialog(QtGui.QDialog):
 			QtCore.SIGNAL("accepted()"),
 			lambda: self.save_instrument())
 
+		for x in range(116):
+			self.ui.minnote.addItem("%s-%d" % (int_to_note(x % 12), x / 12 + 1))
+			self.ui.maxnote.addItem("%s-%d" % (int_to_note(x % 12), x / 12 + 1))
+
+
 	def save_instrument(self):
 		chan = self.ui.channel.value()
 		drum = False
@@ -49,10 +55,25 @@ class InstrumentDialog(QtGui.QDialog):
 			res += "step:%d " % self.ui.step.value()
 		if self.ui.stepend.value() != -1:
 			res += "end:%d " % self.ui.stepend.value()
+		if self.ui.end.value() != -1:
+			res += "global_end:%d " % self.ui.end.value()
 		res += "max_velocity:%d min_velocity:%d " % (self.ui.maxvelocity.value(),
 				self.ui.minvelocity.value())
-		res += "note_length:%d " % self.ui.noteduration.value()
+
+		min_length = self.ui.minnoteduration.value()
+		max_length = self.ui.noteduration.value()
+		if min_length > max_length:
+			a = min_length
+			min_length = max_length
+			max_length = a
+
+		res += "note_length:%d " % max_length
+		
+		if min_length != max_length:
+			res += "min_note_length:%d " % self.ui.minnoteduration.value()
 		res += "max_notes:%d " % self.ui.maxnotes.value()
+		res += "min_note:%d " % self.ui.minnote.currentIndex()
+		res += "max_note:%d " % self.ui.maxnote.currentIndex()
 		res += "let_ring:1 " # for backwards compatibility
 		res += "}"
 		self.item.setText(res)
@@ -70,17 +91,8 @@ class InstrumentDialog(QtGui.QDialog):
 		self.ui.stepstart.setValue(i.start)
 		self.ui.stepend.setValue(i.end)
 		self.ui.step.setValue(i.step)
-		if 'max_velocity' in i.params:
-			self.ui.maxvelocity.setValue(i.params["max_velocity"])
-		else:
-			self.ui.maxvelocity.setValue(100)
-		if 'min_velocity' in i.params:
-			self.ui.minvelocity.setValue(i.params["min_velocity"])
-		else:
-			self.ui.minvelocity.setValue(50)
+		self.ui.end.setValue(i.global_end)
 
-		if 'chance' in i.params:
-			self.ui.chance.setValue(i.params['chance'])
 
 		self.ui.channel.setEnabled(True)
 		if 'channel' in i.params:
@@ -90,20 +102,25 @@ class InstrumentDialog(QtGui.QDialog):
 		else:
 			self.ui.channel.setValue(0)
 
+
 		if 'note_length' in i.params:
 			self.ui.noteduration.setValue(i.params["note_length"])
+			if 'min_note_length' not in i.params:
+				self.ui.minnoteduration.setValue(i.params["note_length"])
 		else:
-			self.ui.noteduration.setValue(2)
+			self.ui.noteduration.setValue(1)
 
-		if 'midi_instr' in i.params:
-			self.ui.midi.setCurrentIndex(i.params['midi_instr'])
-		else:
-			self.ui.midi.setCurrentIndex(0)
+		if 'chance' in i.params:
+			self.ui.chance.setValue(i.params['chance'])
+
+		self.ui.maxvelocity.setValue(i.params["max_velocity"]) if 'max_velocity' in i.params else self.ui.maxvelocity.setValue(100)
+		self.ui.minvelocity.setValue(i.params["min_velocity"]) if 'min_velocity' in i.params else self.ui.minvelocity.setValue(50)
+		self.ui.minnoteduration.setValue(i.params['min_note_length']) if 'min_note_length' in i.params else self.ui.maxnotes.setValue(1)
+		self.ui.midi.setCurrentIndex(i.params['midi_instr']) if 'midi_instr' in i.params else self.ui.midi.setCurrentIndex(0)
+		self.ui.maxnotes.setValue(i.params['max_notes']) if 'max_notes' in i.params else self.ui.maxnotes.setValue(-1)
+		self.ui.minnote.setCurrentIndex(i.params['min_note']) if 'min_note' in i.params else self.ui.minnote.setCurrentIndex(0)
+		self.ui.maxnote.setCurrentIndex(i.params['max_note']) if 'max_note' in i.params else self.ui.maxnote.setCurrentIndex(115)
 
-		if 'max_notes' in i.params:
-			self.ui.maxnotes.setValue(i.params['max_notes'])
-		else:
-			self.ui.maxnotes.setValue(-1)
 
 
 def combo_index_by_text(combo, text):
