@@ -81,13 +81,18 @@ class MovementScene(QtGui.QGraphicsScene):
 		self.pressed = []
 
 	def plays(self, params, n):
-		start, end, step = -1, -1, -1
+		start, end, step, global_end = -1, -1, -1, -1
 		if 'start' in params:
 			start = params['start']
 		if 'step' in params:
 			step = params['step']
 		if 'end' in params:
 			end = params['end']
+		if 'global_end' in params:
+			global_end = params['global_end']
+
+		if global_end != -1 and n >= global_end:
+			return False
 
 		if start == -1:
 			return True
@@ -255,6 +260,15 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 		self.connect(self.ui.addblockbutton,
 			QtCore.SIGNAL("clicked()"),
 			self.add_block)
+		self.connect(self.ui.instruments,
+			QtCore.SIGNAL("currentItemChanged(QListWidgetItem*, QListWidgetItem*)"),
+			lambda x,y: self.instrument_changed())
+		self.connect(self.ui.blocks,
+			QtCore.SIGNAL("currentItemChanged(QListWidgetItem*, QListWidgetItem*)"),
+			lambda x,y: self.block_changed())
+		self.connect(self.ui.progressions,
+			QtCore.SIGNAL("currentItemChanged(QListWidgetItem*, QListWidgetItem*)"),
+			lambda x,y: self.progression_changed())
 		self.connect(self.ui.addprogressionbutton,
 			QtCore.SIGNAL("clicked()"),
 			self.add_progression)
